@@ -11,7 +11,8 @@ import pyutilib.component.core
 pyutilib.component.core.PluginGlobals.push_env("pyutilib.workflow")
 
 from globals import *
-from connections import *
+from connector import *
+from port import *
 from resource import *
 from task import *
 from workflow import *
