@@ -7,16 +7,24 @@
 #  the U.S. Government retains certain rights in this software.
 #  _________________________________________________________________________
 
+"""Definitions for workflow task objects."""
+
 __all__ = ['Task', 'EmptyTask', 'Component']
 
-import connections
+import port
 from options import OptionParser
 import globals
+from pyutilib.misc import Options
+import pprint
 
 
 class Task(object):
+    """
+    A Task object represents a single action in a workflow.
+    """
 
     def __init__(self, id=None, name=None):
+        """Constructor."""
         if not id is None:
             self.id = id
         else:
@@ -25,51 +33,63 @@ class Task(object):
             self.name = "Task"+str(self.id)
         else:
             self.name = name
-        self.inputs = connections.InputConnections(self)
+        self.inputs = port.InputPorts(self)
         self.inputs.set_name(self.name+"-inputs")
-        self.outputs = connections.OutputConnections(self)
+        self.outputs = port.OutputPorts(self)
         self.outputs.set_name(self.name+"-outputs")
         self._resources = {}
         self._predecessors = []
         self._create_parser()
 
-    def add_resource(self, res):
-        self._resources[res.name] = res
+    def add_resource(self, resource):
+        """Add a resource that is required for this task to execute."""
+        self._resources[resource.name] = resource
 
     def resource(self, name):
+        """Return the specified resource object."""
         return self._resources[name]
 
     def next_tasks(self):
-        return set(t for name in self.outputs for t in self.outputs[name].to_task)
+        """Return the set of tasks that succeed this task in the workflow."""
+        return set(t.to_port.task for name in self.outputs for t in self.outputs[name].output_connections)
             
     def prev_tasks(self):
-        return set(self.inputs[name].from_task for name in self.inputs if self.inputs[name].from_task.id != NoTask.id) | set(task for task in self._predecessors)
+        """Return the set of tasks that precede this task in the workflow."""
+        return set([task for name in self.inputs for task in self.inputs[name].from_tasks() if task.id != NoTask.id]) | set(task for task in self._predecessors)
            
     def next_task_ids(self):
+        """Return the set of ids for tasks that succeed this task in the workflow."""
         return set(task.id for task in self.next_tasks())
             
     def prev_task_ids(self):
+        """Return the set of ids for tasks that precede this task in the workflow."""
         return set(task.id for task in self.prev_tasks())
            
     def execute(self):
+        """Execute this task."""
         raise ValueError, "There is no default execution for an abstract Task object! Task=%s" % str(self) #pragma:nocover
 
     def busy(self):
+        """Return the list of resources that this task is waiting for."""
         return [name for name in self._resources if not self._resources[name].available()]
 
     def __call__(self, *options, **kwds):
-        """ 
-        Copy the inputs into this Task's dictionary, 
-        then execute the task, then copy 
+        """Setup inputs and output parameters and execute this task.
+
+        Copy the inputs into this Task's dictionary, then execute the task, then copy 
         the outputs out of the dictionary.
         """
+        self._call_init(*options, **kwds)
+        self.execute()
+        return self._call_fini(*options, **kwds)
+
+    def _call_init(self, *options, **kwds):
         self._call_start()
         busy = self.busy()
         if len(busy) > 0:
             raise IOError, "Cannot execute task %s.  Busy resources: %s" % (self.name, str(busy))
         # Set inputs
         for opt in options:
-            #print "Y",opt
             self._set_inputs(opt)
         self._set_inputs(kwds)
         #
@@ -80,16 +100,26 @@ class Task(object):
             setattr(self, i, None)
         for i in self.inputs:
             #print "X",i,self.inputs[i].get_value(),str(self.inputs[i])
+            # TODO: validate that non-optional inputs have a value other than None
+            self.inputs[i].compute_value()
             setattr(self, i, self.inputs[i].get_value())
-        self.execute()
+
+    def _call_fini(self, *options, **kwds):
         for i in self.outputs:
             #print "Z",i,getattr(self.outputs,i).get_value()
+            # TODO: validate that non-optional outputs have a value other than None
             self.outputs[i].set_value( getattr(self, i) )
         for name, res in self._resources.iteritems():
             res.unlock()
         self._call_finish()
+        #
+        opt = Options()
+        for i in self.outputs:
+            setattr(opt, i, getattr(self.outputs,i).get_value())
+        return opt
 
     def set_options(self, args):
+        """Use a list of command-line options to initialize this task."""
         [self.options, args] = self.parser.parse_args(args)
         tmp = {}
         for opt in self.parser._long_opt:
@@ -101,19 +131,20 @@ class Task(object):
         self._set_inputs(tmp)
 
     def _call_start(self):
+        """This method is executed when the task is started."""
         pass
 
     def _call_finish(self):
+        """This method is executed when the task is finished."""
         pass
 
     def _set_inputs(self, options):
+        """Set the inputs from a dictionary."""
         for key in options:
             self.inputs[key].set_value(options[key])
 
     def _create_parser(self):
-        """
-        Create the OptionParser object and populated it with option groups.
-        """
+        """Create the OptionParser object and populated it with option groups."""
         self.parser = OptionParser()
         self._parser_group = {}
         self._create_parser_groups()
@@ -122,35 +153,46 @@ class Task(object):
             self.parser.add_option_group(self._parser_group[key])
 
     def _create_parser_groups(self):
-        """
-        This method is called by the _create_parser method to setup the
-        parser groups that are registered for this task.
-        """
+        """This method is called by the _create_parser method to setup the
+        parser groups that are registered for this task."""
+
+    def _repn_(self):
+        tmp = {}
+        tmp['A_TYPE'] = 'Task'
+        tmp['Name'] = self.name
+        tmp['Id'] = self.id
+        tmp['Inputs'] = self.inputs._repn_()
+        tmp['Outputs'] = self.outputs._repn_()
+        return tmp
 
     def __repr__(self):
-        return str(self)
+        """Return a string representation for this task."""
+        return pprint.pformat(self._repn_(), 2)
 
     def __str__(self):
+        """Return a string representation for this task."""
         return "%s prev: %s next: %s resources: %s" % (str(self.name),str(sorted(list(self.prev_task_ids()))),str(sorted(list(self.next_task_ids()))), str(sorted(self._resources.keys())))
 
 
-"""
-Alias for the Task class
-"""
 class Component(Task):
+    """
+    Alias for the Task class.
+    """
 
     def __init__(self, *args, **kwds):
+        """Constructor."""
         Task.__init__(self, *args, **kwds)
 
 
 class EmptyTask(Task):
 
     def __init__(self, id=None, name=None):
+        """Constructor."""
         Task.__init__(self, id=None, name=None)
 
     def __call__(self, *args, **kwds):
-        pass
-
+        """Empty task execution."""
 
+# A task instance that represents no task.
 NoTask = Task(id=0)
 
