@@ -7,9 +7,10 @@
 #  the U.S. Government retains certain rights in this software.
 #  _________________________________________________________________________
 
-__all__ = ['TaskPlugin', 'TaskFactory']
+__all__ = ['TaskPlugin', 'TaskFactory', 'WorkflowPlugin']
 
 import task
+import workflow
 from pyutilib.component.core import *
 
 
@@ -29,6 +30,15 @@ class TaskPlugin(Plugin, task.Task):
         task.Task.__init__(self, *args, **kwds)
 
 
+class WorkflowPlugin(Plugin, workflow.Workflow):
+
+    implements(IWorkflowTask)
+
+    def __init__(self, *args, **kwds):
+        Plugin.__init__(self, *args, **kwds)
+        workflow.Workflow.__init__(self, *args, **kwds)
+
+
 class Selection_Task(TaskPlugin):
 
     alias('workflow.selection')
@@ -42,7 +52,5 @@ class Selection_Task(TaskPlugin):
         self.outputs.declare('selection')
 
     def execute(self, *args, **kwds):
-        print self.inputs
-        print self.outputs
         self.selection = self.data[self.index]
 
